package com.iamberry.wechat.service.wechat;

import com.iamberry.app.tool.log.RatFWLogger;
import com.iamberry.wechat.core.entity.member.Member;
import com.iamberry.wechat.core.entity.wx.*;
import com.iamberry.wechat.face.customer.CustomerService;
import com.iamberry.wechat.face.home.HomeService;
import com.iamberry.wechat.face.wechat.*;
import com.iamberry.wechat.service.MessageManager;
import com.iamberry.wechat.service.MessageUtil;
import com.iamberry.wechat.tools.*;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * description ： wechat handler的核心服务类
 * @author 何秀刚
 * create date：2016年3月10日
 */
@Service
public class WeChatServiceImpl implements WeChatService {

	private Logger logger = LoggerFactory.getLogger(WeChatServiceImpl.class);

	@Autowired
	private KeyWordService keyWordService;
	@Autowired
	private RatFWLogger inLongLogger;
	@Autowired
	private MenuKeySetService menuKeySetService;
	@Autowired
	private ImageTextInfoService imageTextInfoService;
	@Autowired
	private ImageTextMoreService imageTextMoreService;
	@Autowired
	private HomeService homeService;
	@Autowired
	private CustomerService customerService;

	/**
	 * 处理微信发来的请求
	 * @param FromUserName
	 * @return
	 */
	@Override
	public String processRequest(String FromUserName, String ToUserName, String MsgType, String CreateTime, String Content, String Event, String Ticket, String EventKey) {
		// 创建Map，防止错误
		Map<String, String> requestMap = new HashMap<String, String>();
		requestMap.put("FromUserName", FromUserName);
		requestMap.put("ToUserName", ToUserName);
		requestMap.put("MsgType", MsgType);
		requestMap.put("CreateTime", CreateTime);
		requestMap.put("Content", Content);
		requestMap.put("Event", Event);
		requestMap.put("Ticket", Ticket);
		requestMap.put("EventKey", EventKey);

		String respMessage = null;
		// 发送方帐号(open_id)
		String fromUserName = null;
		// 公众帐号
		String toUserName = null;
		// 时间
		long createTime = 0L;
		// 消息类型
		String msgType = null;
		try {
			// 默认返回的文本消息内容
			String respContent = "";
			// 解析参数
			fromUserName = requestMap.get("FromUserName");
			toUserName = requestMap.get("ToUserName");
			msgType = requestMap.get("MsgType");
			createTime = System.currentTimeMillis();

			// 准备返回给微信服务器的文本消息对象
			TextMessage textMessage = new TextMessage();
			textMessage.setCreateTime(createTime);
			textMessage.setToUserName(fromUserName);
			textMessage.setFromUserName(toUserName);
			textMessage.setMsgType(MessageUtil.RESP_MESSAGE_TYPE_TEXT);
			//多客服处理
			String intxml = "<xml>"+
					"<ToUserName><![CDATA[" + fromUserName + "]]></ToUserName>" +
					"<FromUserName><![CDATA[" + toUserName + "]]></FromUserName>" +
					"<CreateTime>" + createTime + "</CreateTime>" +
					"<MsgType><![CDATA[transfer_customer_service]]></MsgType></xml>";
			/*
			 * 各种消息处理
			 */
			if(msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_TEXT)){
				//文本消息
				// 消息为空的处理
				String content = requestMap.get("Content");
				if(!(content != null && !"".equals(content))) {
					respContent = ResultInfo.messageIsNullResponseText;
				}
				//去空格
				content = requestMap.get("Content").trim();

				// QQ表情的处理
				if(QQFace.isQqFace(content)){
					return "<xml>"+
							"<ToUserName><![CDATA[" + fromUserName + "]]></ToUserName>" +
							"<FromUserName><![CDATA[" + toUserName + "]]></FromUserName>" +
							"<CreateTime>" + createTime + "</CreateTime>" +
							"<MsgType><![CDATA[transfer_customer_service]]></MsgType></xml>";
				}

				// 激活码为“CODE”开头 二维码在此处理

				// 文本类型的消息处理
				Status status = messageHandler(content, requestMap, fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return respContent = status.getCountext();
				} else {
					respContent = status.getCountext();
				}
			} else if(msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_IMAGE)){
				// 图片消息
				Status status = joinUpCustomer(fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return status.getCountext();
				} else {
					respContent = status.getCountext();
				}
			} else if (msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_LOCATION)) {
				// 地理位置消息
				Status status = joinUpCustomer(fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return status.getCountext();
				} else {
					respContent = status.getCountext();
				}
			} else if (msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_LINK)) {
				// 链接消息
				Status status = joinUpCustomer(fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return status.getCountext();
				} else {
					respContent = status.getCountext();
				}
			} else if (msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_VIDEO)) {
				// 视频消息
				Status status = joinUpCustomer(fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return status.getCountext();
				} else {
					respContent = status.getCountext();
				}
			} else if (msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_VOICE)) {
				// 音频消息
				Status status = joinUpCustomer(fromUserName, toUserName, createTime+"");
				if (status.isFlag()) {
					return status.getCountext();
				} else {
					respContent = status.getCountext();
				}			} else if (msgType.equals(MessageUtil.REQ_MESSAGE_TYPE_EVENT)) {
				// 事件推送
				String eventType = requestMap.get("Event");
				String eventKey = requestMap.get("EventKey");
				// 事件列表：CLICK(点击自定义菜单)/subscribe(关注)/unsubscribe(取消关注)/SCAN(扫描二维码)/LOCATION(上报地理位置)
				if(eventType.equals("SCAN")){

					//扫描机器二维码返回跳转到完善信息页面
					if (eventKey.contains(ResultInfo.barCodePrefix)) {
						Status status = scannQrcodeGiveGift(requestMap, fromUserName);
						if (status.isFlag()) {
							return status.getCountext();
						}
						respContent = status.getCountext();
					}else if(eventKey.contains(ResultInfo.barCodeActivity)){
						Status status = bindingActivity(requestMap, fromUserName,true);
						if (status.isFlag()) {
							return status.getCountext();
						}
						respContent = status.getCountext();
					}else if(eventKey.contains(ResultInfo.barCodeMainChannelId)){
						Status status = bindingMainChannelId(requestMap, fromUserName,false);
						if (status.isFlag()) {
							return status.getCountext();
						}
						respContent = status.getCountext();
					} else {
						// 已关注用户扫描二维码事件
						int flag = subUserScannQrcodeEvent(requestMap, fromUserName);
						if (flag == 1 ) {
							respContent = ResultInfo.subDefaultResponseText;
						} else if ( flag == 0){
							respContent = ResultInfo.subDefaultResponseText;
						} else if (flag == 4) {
							respContent = "欢迎您关注美国watero！您已享受购买水机终身优惠价3280元（普通会员销售价3680元)，并且购买即赠三年滤芯套装，快来购买吧！<a href=\"http://w.iamberry.com/watero/wechat/wxCart/goProductList\">点击购买</a>";
						} else if (flag == 5) {
							//618好友助力
							respContent = "哇！确认过眼神，你将获得我们618天猫上朵旗舰店隐藏福利！</br>";
						}else {
							respContent = "欢迎您关注美国watero，恭喜您成为金牌会员。自己购买或者推荐朋友购买水机均将享受3280元优惠价，并免费赠送3年滤芯哦！点击菜单“分销中心-代理分销”可进入您的专属分销管理中心。";
						}
					}
				} else if (eventType.equals(MessageUtil.EVENT_TYPE_SUBSCRIBE)) {
					// 订阅
					if(requestMap.get("Ticket") != null){

						//扫描机器二维码返回跳转到完善信息页面
						if (eventKey.contains(ResultInfo.barCodePrefix)) {
							Status status = scannQrcodeSubGiveGift(requestMap, fromUserName);
							if (status.isFlag()) {
								return status.getCountext();
							}
							respContent = status.getCountext();
						} else if(eventKey.contains(ResultInfo.barCodeActivity)){
							Status status = bindingActivity(requestMap, fromUserName,false);
							if (status.isFlag()) {
								return status.getCountext();
							}
							respContent = status.getCountext();
						}else if(eventKey.contains(ResultInfo.barCodeMainChannelId)){
							Status status = bindingMainChannelId(requestMap, fromUserName,false);
							if (status.isFlag()) {
								return status.getCountext();
							}
							respContent = status.getCountext();
						} else {
							//未关注用户扫描二维码关注事件处理
							Status status = scannQRcodeSub(requestMap, fromUserName);
							Member member = refreshUserInfo(fromUserName);
							if (status.isFlag()) {
								return status.getCountext();
							}
							respContent = status.getCountext();
						}
					} else {
						// 搜索公众号关注
						Status status = queryWechatSub(fromUserName, requestMap);
						Member member = refreshUserInfo(fromUserName);
						if (status.isFlag()) {
							return status.getCountext();
						}
						respContent = status.getCountext();
					}
				} else if (eventType.equals(MessageUtil.EVENT_TYPE_UNSUBSCRIBE)) {
					// 取消订阅

					unSubScribe(fromUserName);
					respContent = "Success";
				} else if (eventType.equals(MessageUtil.EVENT_TYPE_CLICK) || eventType.equals("VIEW")) {
					// 自定义菜单点击事件
					Status status = clickWechatMenuEnent(requestMap);
					Member member = refreshUserInfo(fromUserName);
					if (status.isFlag()) {
						return status.getCountext();
					}
					respContent = status.getCountext();
				}
			}
			// 回复文本消息
			textMessage.setContent(respContent);
			respMessage = MessageUtil.textMessageToXml(textMessage);
		} catch (Exception e) {
			inLongLogger.debug(this, e.getMessage());
			TextMessage textMessage = new TextMessage();
			textMessage.setCreateTime(createTime);
			textMessage.setToUserName(fromUserName);
			textMessage.setFromUserName(toUserName);
			textMessage.setMsgType(MessageUtil.RESP_MESSAGE_TYPE_TEXT);
			textMessage.setContent(ResultInfo.tryCatchDefaultResponseText);
			respMessage = MessageUtil.textMessageToXml(textMessage);
		}
		return respMessage;
	}

	/**
	 * 618h好友助力
	 * @param requestMap
	 */
	public Status bindingActivity(Map<String, String> requestMap, String fromUserName,Boolean isSubscription) {
		Status status = new Status();
		String senceid = requestMap.get("EventKey");

		status.setCountext(senceid);
		status.setFlag(false);
		return status;
	}


	/**
	 * 渠道关注统计
	 * @param requestMap
	 */
	public Status bindingMainChannelId(Map<String, String> requestMap, String fromUserName,Boolean isSubscription) {
		Status status = new Status();
		String senceid = requestMap.get("EventKey");
		senceid = senceid.replaceAll(ResultInfo.barCodeMainChannelId,"");
		if(!isSubscription){
			senceid = senceid.replaceAll(ResultInfo.barCode,"");
		}
		status.setFlag(false);
		status.setCountext(ResultInfo.subDefaultResponseText);
		return status;
	}

	/**
	 * 扫码进入赠送礼品信息页面（已关注）
	 * @param requestMap
	 */
	public Status scannQrcodeGiveGift(Map<String, String> requestMap, String fromUserName) {
		Status status = new Status();
		String senceid = requestMap.get("EventKey");
		senceid = senceid.replaceAll(ResultInfo.barCodePrefix,"");
		String str = bindingMachine(senceid, fromUserName);
		status.setCountext(str);
		status.setFlag(false);
		return status;
	}

	/**
	 * 扫码进入赠送礼品信息页面（未关注）
	 * @param requestMap
	 * @param fromUserName
	 */
	public Status scannQrcodeSubGiveGift(Map<String, String> requestMap, String fromUserName) {
		//赋值&更新数据
		Status status = new Status();
		String senceid = requestMap.get("EventKey");
		senceid = senceid.replaceAll(ResultInfo.barCodePrefix,"");
		String str = bindingMachine(senceid, fromUserName);
		status.setCountext(str);
		status.setFlag(false);
		return status;
	}

	/**
	 * 上海tooth扫码绑定机器关系
	 * @param senceid
	 * @param fromUserName
	 * @return
	 */
	public String bindingMachine(String senceid, String fromUserName) {
		StringBuilder str = new StringBuilder();
		String url = NameUtils.getConfig("rst_base_url") + senceid;
		Map<String,String> params = new HashMap<String,String>();
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		try {
			//查询百胜订单信息
			String resultJson = HttpClient431Util.doGet(params,url,null,null,null);
			// unicode
			resultJson = StringEscapeUtils.unescapeJava(resultJson);
			JSONObject jsonObject = JSONObject.fromObject(resultJson);
			if (jsonObject.getInt("resultCode") != 200) {
				str.append("没有查到该机器信息，请确认二维码是否正确!");
				return str.toString();
			}
			JSONObject jsonMessage = jsonObject.getJSONObject("message");
			if (jsonMessage.getInt("isBeing") == 2) {
				str.append("没有查到该机器信息，请确认二维码是否正确!");
				return str.toString();
			}
			str.append("您的<");
			str.append(jsonMessage.getString("productType"));
			str.append(">为正品，生产日期为：");
			str.append(format.format(new Date(jsonMessage.getLong("producedTime"))));
			str.append("。请放心使用。");
			return str.toString();
		} catch (Exception e) {
			str.append("没有查到该机器信息，请确认二维码是否正确!");
			return str.toString();
		}
	}

	/**
	 * 已关注用户扫描二维码事件
	 * @param requestMap
	 * @param fromUserName
	 * @throws Exception
	 */
	public int subUserScannQrcodeEvent(Map<String, String> requestMap, String fromUserName) throws Exception {
		//已经关注用户扫描二维码触发事件：获取二维码场景值
		String senceid = requestMap.get("EventKey");
		Member member = refreshUserInfo(fromUserName);
		// 是否更新数据
		boolean isNew = false;
		//没绑关系、没有来源、上线是代理商时的id为空、没有微代理层关系 时， 则可以绑定用户关系
		if (member.getUserIsFlag() != null && member.getUserIsFlag() == 1) {
			isNew = true;
		}
		return buildRelation(member, senceid, isNew);
	}

	/**
	 * 未关注用户扫描二维码关注
	 * @param requestMap
	 * @param fromUserName
	 * @return
	 * @throws Exception
	 */
	public Status scannQRcodeSub(Map<String, String> requestMap, String fromUserName) throws Exception {
		// 解析场景值
		String senceid = requestMap.get("EventKey");
		//赋值&更新数据
		Member member = refreshUserInfo(fromUserName);
		// 是否更新数据
		boolean isNew = false;

		if (member.getUserIsFlag() != null && member.getUserIsFlag() == 1) {
			isNew = true;
		}
		int isMessage = buildRelation(member, senceid, isNew);
		Status status = new Status();
		if (isMessage == 1) {
			status.setFlag(false);
			status.setCountext(ResultInfo.subDefaultResponseText);
		}else if(isMessage == 0){
			status.setFlag(false);
			status.setCountext(ResultInfo.subDefaultResponseText);
		} else if (isMessage == 4) {
			status.setFlag(false);
			status.setCountext("欢迎您关注美国watero！您已享受购买水机终身优惠价3280元（普通会员销售价3680元)，并且购买即赠三年滤芯套装，快来购买吧！<a href=\"http://w.iamberry.com/watero/wechat/wxCart/goProductList\">点击购买</a>");
		} else{
			status.setFlag(false);
			status.setCountext("欢迎您关注美国watero，恭喜您成为金牌会员。自己购买或者推荐朋友购买水机均将享受3280元优惠价，并免费赠送3年滤芯哦！点击菜单“分销中心-代理分销”可进入您的专属分销管理中心。");
		}
		//返回数据
		return status;
	}

	/**
	 * 用户自己搜索微信公众号关注
	 * @param fromUserName
	 * @param requestMap
	 * @return
	 * @throws Exception
	 */
	public Status queryWechatSub(String fromUserName, Map<String, String> requestMap) throws Exception {
		//赋值&更新数据
		refreshUserInfo(fromUserName);
		Status status = new Status();
		String respContent = ResultInfo.subDefaultResponseText;
		status.setCountext(respContent);
		return status;
	}

	/**
	 * 更新微信用户信息
	 * 2016年11月17日
	 */
	private Member refreshUserInfo(String openId){
		System.out.println("====更新用户信息"+openId);
		JSONObject jsonObject = WeixinUtil.getUserInfo(openId);
		String sex = jsonObject.getString("sex");
		String city = jsonObject.getString("city");
		String country = jsonObject.getString("country");
		String province = jsonObject.getString("province");
		String nickname = jsonObject.getString("nickname");
		String headimgurl = jsonObject.getString("headimgurl");

		Member member = homeService.selectMemberInfoByOpenId(openId);
		//赋值&更新会员数据
		if (member != null && member.getUserId() != null) {
			member.setUserOpenid(openId);
			member.setUserNickname(nickname);
			member.setUserSex(Integer.parseInt(sex));
			member.setUserAddr(country+"-"+province+"-"+city);
			member.setUserHead(headimgurl);
			//userIsBought 暂存旧的关注状态
			member.setUserIsBought(member.getUserStatus() == null ? 1 : member.getUserStatus());
			member.setUserStatus(2);
			homeService.updateWechatMemberInfoByByOpenid(member);
		} else {
			member = new Member();
			member.setUserNickname(nickname);
			member.setUserSex(Integer.parseInt(sex));
			member.setUserAddr(country+"-"+province+"-"+city);
			member.setUserHead(headimgurl);
			member.setUserStatus(2);
			member.setUserOpenid(openId);
			member.setUserIsBought(1);
			member.setUserIsFlag(1);
			homeService.insertMemberInfo(member);
		}
		return member;
	}

	/**
	 * 绑定关系
	 * @param member	当前用户
	 * @param senceid	二维码场景值
	 * 					两种：1.代理商的永久二维码 <格式:qrscene_QR***>
	 * 						2.微代理的临时二维码 <格式:qrscene_***>
	 * 					其中 *** 是二维码表（或是临时二维码表）id
	 * @param isNew		是否更新关系
	 * @return	0：默认	1：绑定成功		2：金牌会员
	 */
	public int buildRelation(Member member,String senceid,boolean isNew){

		int isMessage = 0;
		return isMessage;
	}
	/**
	 * 用户取消关注时的事件
	 * @param fromUserName
	 */
	public void unSubScribe(String fromUserName) {
		try {
			// 更新用户状态
			homeService.updateMemberStatusByOpenId(fromUserName);
			// 更新渠道二维码信息,如果有
			// qrcodeService.updateQRcodeStatusByUserOpenid(fromUserName);
			// 待确认，是否微代理一旦取消关注就自动回收微代理的资格?
		} catch (Exception e) {
			inLongLogger.error(this, e.getMessage());
		}
	}

	/**
	 * 自定义菜单点击事件
	 * @param requestMap
	 * @return
	 * @throws Exception
	 */
	public Status clickWechatMenuEnent(Map<String, String> requestMap) throws Exception {
		Status status = new Status();
		// 事件KEY值,与创建自定义菜单时指定的KEY值对应 
		String eventKey = requestMap.get("EventKey").trim();
		//保存消息记录
		MenuKeySet menuKeySet = menuKeySetService.selectByPubNoAndKeyWord(NameUtils.getConfig("pubNo"), eventKey);
		String respContent = ResultInfo.subDefaultResponseText;
		if(menuKeySet != null){
			if("1".equals(menuKeySet.getReType())){
				//文本
				respContent = menuKeySet.getRefText();
			}else if("2".equals(menuKeySet.getReType())){
				//图文
				List<ImageTextMore> imagetextMoreList = null;
				ImageTextInfo imageTextInfo = imageTextInfoService.findImageTextInfoById(Long.parseLong(menuKeySet.getRefImageTextId()));
				if(imageTextInfo != null) {
					//判断是否多图文
					if("2".equals(imageTextInfo.getImageTextType())){
						imagetextMoreList = imageTextMoreService.findImageTextmore(imageTextInfo.getImageTextNo());
					}
					status.setFlag(true);
					respContent = MessageManager.getImageTextInfo(requestMap, imageTextInfo, imagetextMoreList);
				}
			}
		}
		status.setCountext(respContent);
		return status;
	}

	/***
	 * 消息处理
	 * @param content
	 * @param requestMap
	 * @param fromUserName
	 * @param toUserName
	 * @param createTime
	 * @return
	 */
	public Status messageHandler(String content, Map<String, String> requestMap, String fromUserName, String toUserName, String createTime) {
		Status status = new Status();
		status.setCountext(ResultInfo.messageDefaultResponseText);
		status.setFlag(true);
		KeyWord keyword = null;
		try {
			//是否命中关键字
			keyword = keyWordService.getByKeyWord(content);
		} catch (Exception e) {
			inLongLogger.info(e.getMessage() + "," + content);
		}
		if(keyword != null) {
			//判断是否存在关键字回复
			if("1".equals(keyword.getReType())) {
				//文本
				status.setCountext(keyword.getRefText());
				status.setFlag(false);
			} else if("2".equals(keyword.getReType())){
				//图文
				List<ImageTextMore> imagetextMoreList = null;
				ImageTextInfo imageTextInfo = imageTextInfoService.findImageTextInfoById(keyword.getRefImageTextId());
				if(imageTextInfo != null) {
					//判断是否为多图文
					if("2".equals(imageTextInfo.getImageTextType())){
						//判断是否多图文
						imagetextMoreList = imageTextMoreService.findImageTextmore(imageTextInfo.getImageTextNo());
					}
					try {
						String s = MessageManager.getImageTextInfo(requestMap, imageTextInfo, imagetextMoreList);
						status.setCountext(s);
						status.setFlag(true);
						return status;
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		} else {
			// 如果没有命中关键字，直接接入到客服
			return joinUpCustomer(fromUserName, toUserName, createTime);
		}
		return status;
	}

	public Status joinUpCustomer(String fromUserName, String toUserName, String createTime) {
		Status result = new Status();
		// 查询当前是否有客服在线
		int num = customerService.count(1);
		if (num >= 1) {
			String xml = "<xml>"+
					"<ToUserName><![CDATA[" + fromUserName + "]]></ToUserName>" +
					"<FromUserName><![CDATA[" + toUserName + "]]></FromUserName>" +
					"<CreateTime>" + createTime + "</CreateTime>" +
					"<MsgType><![CDATA[transfer_customer_service]]></MsgType></xml>";
			result.setCountext(xml);
			result.setFlag(true);
			return result;
		}
		// 判断客服是否离开
		num = customerService.count(2);
		if (num >= 1) {
			result.setCountext("尊敬的用户您好，我们每天的服务时段为：9：00-12：00；14：00-18：00，如您在非服务时段内咨询，我们将在看到后第一时间答复，请您耐心等待。");
			result.setFlag(false);
			return result;
		}
		// 客服下线
		result.setCountext("尊敬的用户您好，我们每天的服务时段为 9：00-18：00，如您在非服务时段内咨询，我们将在看到后第一时间答复；您也可拨打上朵全国客户服务热线：400-678-1860");
		result.setFlag(false);
		return result;
	}

}